package com.technology.huahai.web.controller.csm;

import org.springframework.core.io.FileSystemResource;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.font.FontRenderContext;
import java.awt.font.LineBreakMeasurer;
import java.awt.font.TextAttribute;
import java.awt.font.TextLayout;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.text.AttributedString;

@RestController
@RequestMapping("/technology")
public class CsmFileController {

    @GetMapping("/getImage")
    public ResponseEntity<Resource> getImage(@RequestParam("fileName") String fileName) {
        Resource resource = new FileSystemResource(fileName);
        if (!resource.exists()) {
            return ResponseEntity.notFound().build();
        }
        return ResponseEntity.ok()
                .header(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + resource.getFilename() + "\"")
                .body(resource);
    }

    public static void main(String[] args) throws IOException {
        try {
            // 1. 读取原始图片
            File input = new File("D:/00000003.png");
            BufferedImage image = ImageIO.read(input);

            // 2. 创建一个更大的图片空间，用于在底部留出空间写字
            int width = image.getWidth()+60;
            int height = image.getHeight() + 170; // 增加高度用于显示文字

            // 新建一个图片缓冲区，带透明通道可以不设置 TYPE_INT_ARGB
            BufferedImage combined = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
            Graphics2D g2d = combined.createGraphics();

            // 抗锯齿
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            g2d.setColor(new Color(0x4498F0));
            g2d.fillRect(0, 0, width, height);

            // 3. 先画原图
            g2d.drawImage(image, 30, 70, null);

            // 4. 设置字体和颜色
            g2d.setColor(Color.WHITE); // 文字颜色
            Font font = new Font("微软雅黑", Font.BOLD, 24); // 字体样式和大小
            g2d.setFont(font); // 字体样式和大小

            // 要添加的文字
            String text = "这是图片的描述信息111第三个是小地念框架档口上的纽扣考虑公司考虑那个克里斯蒂娜扣篮大赛";

            // 获取文字宽度，实现居中绘制
            FontMetrics fm = g2d.getFontMetrics();
            int textWidth = fm.stringWidth(text);

            // 5. 绘制文字
            // 使用AttributedString处理自动换行
            AttributedString styledText = new AttributedString(text);
            styledText.addAttribute(TextAttribute.FONT, font);
            FontRenderContext frc = g2d.getFontRenderContext();
            LineBreakMeasurer measurer = new LineBreakMeasurer(styledText.getIterator(), frc);
            float wrappingWidth = width - 40; // 减去左右边距
            float y = image.getHeight() + 100; // 文字从图片底部开始的位置

            while (measurer.getPosition() < text.length()) {
                TextLayout layout = measurer.nextLayout(wrappingWidth);
                float x = (float) (width - layout.getBounds().getWidth()) / 2; // 居中
                layout.draw(g2d, x, y);
                y += layout.getAscent() + layout.getDescent() + layout.getLeading(); // 向下移动到下一行的开始位置
            }

            g2d.drawString("请扫码登记", width/2-60, 50);

            // 6. 释放资源
            g2d.dispose();

            // 7. 保存新图片
            File output = new File("D:/00000004.png");
            ImageIO.write(combined, "png", output);

            System.out.println("图片已成功添加文字并保存！");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
