package com.technology.huahai.web.service.impl;


import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.technology.huahai.common.constant.RedisKeyConstant;
import com.technology.huahai.common.core.controller.domain.AjaxResult;
import com.technology.huahai.common.exception.CsmException;
import com.technology.huahai.common.utils.*;
import com.technology.huahai.web.vo.LoginResultVo;
import com.technology.huahai.web.vo.LoginVo;
import com.technology.huahai.web.vo.UserResultVo;
import com.technology.huahai.web.vo.UserVO;
import com.technology.huahai.web.service.SystemLoginService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

@Service
public class SystemLoginServiceImpl implements SystemLoginService {

    protected final Logger logger = LoggerFactory.getLogger(this.getClass());
    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private RedisUtils redisUtils;

    @Value("${zx.userInfoUrl}")
    private String getUserInfoUrl;

    @Value("${zx.secret}")
    private String secret;


    private ObjectMapper objectMapper = new ObjectMapper();

    @Override
    public LoginResultVo getLoginMessage(LoginVo loginVo) {
        try {
            return getUserInfo(loginVo);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    public LoginResultVo getUserInfo(LoginVo loginVo) throws JsonProcessingException {
        LoginResultVo loginResultVo = new LoginResultVo();
        // 设置请求头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);

        // 将请求体和请求头组合到 HttpEntity 中
        HttpEntity<LoginVo> requestEntity = new HttpEntity<>(loginVo, headers);

        // 发送 POST 请求
        ResponseEntity<AjaxResult> response = restTemplate.postForEntity(getUserInfoUrl, requestEntity, AjaxResult.class);
        AjaxResult ajaxResult = response.getBody();
        logger.info("解析票据 {}",ajaxResult);
        if("200".equals(String.valueOf(ajaxResult.getCode()))){
            UserResultVo userResultVo = objectMapper.readValue(objectMapper.writeValueAsString(ajaxResult.getData()), UserResultVo.class);
            if("1".equals(userResultVo.getUserType())){
                //todo 对userstr进行解密
                UserVO userVO = new UserVO();
                String userStr = null;
                try {
                    userStr = Sm4Util.decryptEcb(secret, userResultVo.getUser());
                } catch (Exception e) {
                    e.printStackTrace();
                    throw new CsmException("数据解密失败");
                }
                userVO = JsonUtils.parseJson(userStr, UserVO.class);
                String token="";
                try {
                    token = TokenGeneratorUtils.generateToken(userVO.getCardNo(),userVO.getUserName(),userVO.getMobile());
                    redisUtils.set(RedisKeyConstant.tokenKey +userVO.getCardNo(),token,172800);
                }catch (Exception e){
                    e.printStackTrace();
                    throw new CsmException("解析token失败");
                }
                loginResultVo.setUserName(userVO.getUserName());
                loginResultVo.setMobile(userVO.getMobile());
                loginResultVo.setToken(token);
            }else{
                throw new CsmException("当前用户检测为非个人");
            }
            return loginResultVo;
        }else{
            throw new CsmException("第三方平台票据认证失败");
        }
    }

}
