package com.technology.huahai.common.config.mybaties;

import com.technology.huahai.common.utils.StringUtils;
import org.apache.ibatis.io.VFS;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.mybatis.spring.annotation.MapperScan;
import org.mybatis.spring.boot.autoconfigure.SpringBootVFS;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.core.io.DefaultResourceLoader;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;

import javax.sql.DataSource;


/**
 * Mybatis支持*匹配扫描包
 * 
 * @author taoyuan
 */
@Configuration
@MapperScan(basePackages="com.technology.huahai.web.mapper.coremapper",sqlSessionFactoryRef = "coreSqlSessionFactory")
public class CoreMyBatisConfig extends MybatiesConfig{

    @Autowired
    private Environment env;

    @Autowired
    @Qualifier("coreDataSource")
    DataSource dataSource;

    @Bean(name="coreSqlSessionFactory")
    public SqlSessionFactory sqlSessionFactorya() throws Exception {
        String typeAliasesPackage = env.getProperty("mybatis.typeAliasesPackage");
        String mapperLocations = env.getProperty("mybatis.mapperLocations");
        String configLocation = env.getProperty("mybatis.configLocation");
        //typeAliasesPackage = setTypeAliasesPackage(typeAliasesPackage);
        VFS.addImplClass(SpringBootVFS.class);

        SqlSessionFactoryBean sessionFactory = new SqlSessionFactoryBean();
        sessionFactory.setDataSource(dataSource);
        sessionFactory.setTypeAliasesPackage(typeAliasesPackage);
        sessionFactory.setMapperLocations(resolveMapperLocations(StringUtils.split(mapperLocations, ",")));
        sessionFactory.setMapperLocations(new PathMatchingResourcePatternResolver().getResources("classpath:mapper/coremapper/*.xml"));
        sessionFactory.setConfigLocation(new DefaultResourceLoader().getResource(configLocation));
        return sessionFactory.getObject();
    }

    @Bean(name="coreSqlSessionTemplate")
    SqlSessionTemplate coreSqlSessionTemplate()throws Exception{
        return new SqlSessionTemplate(sqlSessionFactorya());
    }

    @Bean(name="coreTransactionManager")
    public DataSourceTransactionManager coreTransactionManager() {
        return new DataSourceTransactionManager(dataSource);
    }
}