package com.technology.huahai.common.filter;

import com.alibaba.fastjson2.JSON;
import com.technology.huahai.common.constant.RedisKeyConstant;
import com.technology.huahai.common.exception.CsmTokenException;
import com.technology.huahai.common.utils.RedisUtils;
import com.technology.huahai.common.utils.StringUtils;
import com.technology.huahai.common.utils.TokenGeneratorUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Map;
@Configuration
public class JwtTokenFilter extends OncePerRequestFilter {
    @Autowired
    private RedisUtils redisUtils;

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain)
            throws ServletException, IOException {
        String token = resolveToken(request);
        if (token != null) {
            try {
                String str = TokenGeneratorUtils.parseToken(token);
                String redisToken = String.valueOf(redisUtils.get(RedisKeyConstant.tokenKey+str.split(":")[0]));
                if(StringUtils.isEmpty(redisToken) || "null".equals(redisToken) ){
                    getOutException(response,"token 过期");
                    return;
                }
            } catch (CsmTokenException e){
                try {
                    getOutException(response,e.getMessage());
                    return;
                } catch (Exception ex) {
                    throw new RuntimeException(ex);
                }
            }catch (Exception e) {
                try {
                    e.printStackTrace();
                    getOutException(response,"token 验证异常");
                    return;
                } catch (Exception ex) {
                    throw new RuntimeException(ex);
                }
            }
            // 如果需要，这里可以设置认证信息到Spring Security上下文中
        } else {
            try {
                getOutException(response,"token缺失");
                return;
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }

        filterChain.doFilter(request, response);
    }

    private String resolveToken(HttpServletRequest request) {
        String bearerToken = request.getHeader("csm_token");
        if(StringUtils.isBlank(bearerToken)){
            bearerToken = request.getHeader("Authorization");
        }

        if (bearerToken != null && bearerToken.startsWith("Bearer ")) {
            return bearerToken.substring(7); // 提取Bearer后的实际Token
        }
        return null;
    }

    private void getOutException(HttpServletResponse response,String mes) throws Exception {
        Map<String,Object> resultMap = new HashMap<>();
        resultMap.put("msg",mes);
        resultMap.put("code",401);
        response.setContentType("application/json;charset=UTF-8");
        PrintWriter out = response.getWriter();
        String jsonObject = JSON.toJSONString(resultMap);
        out.println(jsonObject);
        out.flush();
        out.close();
    }


}
