package com.technology.huahai.common.utils;

import com.technology.huahai.common.exception.CsmTokenException;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import javax.servlet.http.HttpServletRequest;
import java.nio.charset.StandardCharsets;
import java.util.Base64;

public class TokenGeneratorUtils {

    // 密钥（应保存在配置文件中或环境变量中）
    private static final String SECRET_KEY = "tjcsm02050507";


    /**
     * 生成 Token
     */
    public static String generateToken(String zjhm,String userName,String mobile) throws Exception {
        String payload = zjhm + ":" + userName + ":" + mobile + ":" + System.currentTimeMillis();
        String base64Payload = base64Encode(payload.getBytes(StandardCharsets.UTF_8));

        String signature = calculateSignature(base64Payload);
        return base64Payload + "." + base64Encode(signature.getBytes(StandardCharsets.UTF_8));
    }

    /**
     * 验证并解析 Token
     */
    public static String parseToken(String token) throws Exception {
        String[] parts = token.split("\\.");
        if (parts.length != 2) {
            throw new CsmTokenException("Invalid token format");
        }

        String base64Payload = parts[0];
        String base64Signature = parts[1];

        String expectedSignature = calculateSignature(base64Payload);

        // 比较签名是否一致（注意防止时序攻击，可使用 MessageDigest.isEqual）
        if (!expectedSignature.equals(new String(base64Decode(base64Signature), StandardCharsets.UTF_8))) {
            throw new CsmTokenException("Invalid token signature");
        }

        String payload = new String(base64Decode(base64Payload), StandardCharsets.UTF_8);
        String[] payloadParts = payload.split(":");

        if (payloadParts.length != 4) {
            throw new CsmTokenException("Invalid token payload");
        }

        String zjhm = payloadParts[0];
        String userName = payloadParts[1];
        String mobile = payloadParts[2];

        return zjhm+":"+userName+":"+mobile;
    }

    /**
     * 计算签名使用 HMAC-SHA256 对前一部分数据进行签名
     */
    private static String calculateSignature(String data) throws Exception {
        Mac mac = Mac.getInstance("HmacSHA256");
        SecretKeySpec secretKeySpec = new SecretKeySpec(SECRET_KEY.getBytes(StandardCharsets.UTF_8), "HmacSHA256");
        mac.init(secretKeySpec);
        byte[] rawHmac = mac.doFinal(data.getBytes(StandardCharsets.UTF_8));
        return base64Encode(rawHmac);
    }

    /**
     * Base64 编码
     */
    private static String base64Encode(byte[] data) {
        return Base64.getEncoder().encodeToString(data);
    }

    /**
     * Base64 解码
     */
    private static byte[] base64Decode(String data) {
        return Base64.getDecoder().decode(data);
    }

    public static String getUserMessage(HttpServletRequest httpServletRequest) throws Exception {
        String authorization = httpServletRequest.getHeader("csm_token");
        if(StringUtils.isBlank(authorization)){
            authorization = httpServletRequest.getHeader("Authorization");
        }
        authorization = authorization.substring(7);
        String userMessage = parseToken(authorization);
        return userMessage;
    }

}
