package com.technology.huahai.common.utils.uuid;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * ID生成器工具类
 * 
 * @author taoyuan
 */
public class IdUtils
{

    private static final AtomicInteger sequence = new AtomicInteger(0);
    private static final Object lock = new Object();
    private static String lastDatePart = "";

    /**
     * 获取随机UUID
     * 
     * @return 随机UUID
     */
    public static String randomUUID()
    {
        return UUID.randomUUID().toString();
    }

    /**
     * 简化的UUID，去掉了横线
     * 
     * @return 简化的UUID，去掉了横线
     */
    public static String simpleUUID()
    {
        return UUID.randomUUID().toString(true);
    }

    /**
     * 获取随机UUID，使用性能更好的ThreadLocalRandom生成UUID
     * 
     * @return 随机UUID
     */
    public static String fastUUID()
    {
        return UUID.fastUUID().toString();
    }

    /**
     * 简化的UUID，去掉了横线，使用性能更好的ThreadLocalRandom生成UUID
     * 
     * @return 简化的UUID，去掉了横线
     */
    public static String fastSimpleUUID()
    {
        return UUID.fastUUID().toString(true);
    }

    /**
     * 根据当前时间生成唯一批次号
     * @return
     */
    public static String generateBatchNumber() {
        LocalDateTime now = LocalDateTime.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");
        String datePart = now.format(formatter);

        // 如果进入新的一秒，则重置序列号
        if (!datePart.equals(lastDatePart)) {
            synchronized (lock) {
                if (!datePart.equals(lastDatePart)) {
                    lastDatePart = datePart;
                    sequence.set(0);
                }
            }
        }
        // 生成3位数的序列号（可以调整）
        int seq = sequence.incrementAndGet();
        String sequencePart = String.format("%03d", seq);

        return datePart + sequencePart;
    }

}
